/*

 partition.c - A mostly working PCLink server for IDEDOS 0.9x

 Written by
  Kajtar Zsolt <soci@c64.rulez.org>

    This file is part of IDEDOS the IDE64 disk operating system
    See README for copyright notice.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "partition.h"
#include <string.h>
#include <stdlib.h>
#if defined _WIN32 || defined __CYGWIN__
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#endif

static partition_t work_partition;
static char null_path;
enum { PARTITION_NUM = 26 };


static Partition *partition(partition_t n) {
    static Partition partitions[PARTITION_NUM];
    if (n == 0) n = work_partition;
    if (n < 1 || n > PARTITION_NUM) return NULL;
    return &partitions[n - 1];
}

void partition_create(partition_t n, const Petscii *name, const char *root) {
    Partition *p = partition(n);
    unsigned int i;

    if (p == NULL || p->path != NULL) return;
    for (i = 0; name[i] != '\0' && i < sizeof(p->name) - 1; i++) {
        p->name[i] = name[i];
    }
    p->name[i] = 0;
    p->root = strlen(root);
    p->nr = n;
    if (p->root == 0) {
        p->path = &null_path;
    } else {
        size_t len = p->root + 1;
        p->path = (char *)malloc(len);
        if (p->path == NULL) exit(EXIT_FAILURE);
        memcpy(p->path, root, len);
    }
}

static void partition_delete(partition_t n) {
    Partition *p = partition(n);
    if (p == NULL || p->path == NULL) return;
    if (p->path != &null_path) free(p->path);
    p->path = NULL;
}

void partition_set_current(partition_t n) {
    work_partition = n;
}

Partition *partition_get(partition_t n) {
    Partition *p = partition(n);
    if (p == NULL || p->path == NULL) return NULL;
    return p;
}

partition_t partition_get_current(void) {
    return work_partition;
}

void partition_set_path(Partition *p, const char *path) {
    size_t len;

    if (p == NULL || p->path == NULL) return;
    if (path[0] == 0) {
        if (p->path != &null_path) {
            free(p->path);
            p->path = &null_path;
        }
        return;
    }
    len = strlen(path) + 1 + p->root;
    if (len > strlen(p->path) + 1) {
        char *nm = (char *)(p->path == &null_path ? malloc(len) : realloc(p->path, len));
        if (nm == NULL) return;
        p->path = nm;
    }
    if (len > p->root) {
        memcpy(p->path + p->root, path + p->root, len - p->root);
    } else {
        p->path[p->root] = 0;
    }
}

#if defined _WIN32 || defined __CYGWIN__
static int from_drive_list;
static DWORD old_drives;

void partition_set_auto(void) {
    from_drive_list = 1;
}

void partition_update(void) {
    if (from_drive_list) {
        partition_t current;
        Petscii name[2] = { 'A', 0 };
        char root[4] = { 'A', ':', '/', 0 };
        DWORD drives = GetLogicalDrives() & 0x3ffffff;

        for (current = 0; drives != old_drives && current < 26;) {
            DWORD drive = 1u << current;
            current++;
            if (((drives ^ old_drives) & drive) != 0) {
                old_drives ^= drive;
                if ((drives & drive) == 0) {
                    partition_delete(current);
                } else {
                    name[0] = 'A' - 1 + current;
                    root[0] = name[0];
                    partition_create(current, name, root);
                }
            }
        }
    }
}
#endif

void partition_destroy(void) {
    unsigned int i;
    for (i = 0; i < PARTITION_NUM; i++) {
        partition_delete(i + 1);
    }
}
